@@ -1,3 +1,9 @@
+from __future__ import (
+    absolute_import,
+    print_function,
+    unicode_literals,
+)
+
 from pydocx.DocxParser import DocxParser
 
 
