@@ -1,16 +1,23 @@
 # -*- coding: utf-8 -*-
+from __future__ import (
+    absolute_import,
+    print_function,
+    unicode_literals,
+)
+
 import os
+import sys
 import time
 
 from nose.plugins.skip import SkipTest
 
-from pydocx.DocxParser import OPCPart
 from pydocx.tests.document_builder import DocxBuilder as DXB
 from pydocx.tests import (
     XMLDocx2Html,
     _TranslationTestCase,
 )
 from pydocx.utils import parse_xml_from_string, find_all
+from pydocx.wordml import ImagePart
 
 
 class StyleIsOnTestCase(_TranslationTestCase):
@@ -84,7 +91,7 @@ class StyleIsOnTestCase(_TranslationTestCase):
             ),
         ]
 
-        body = ''
+        body = b''
         for tag in tags:
             body += tag
         xml = DXB.xml(body)
@@ -95,8 +102,8 @@ class HyperlinkVanillaTestCase(_TranslationTestCase):
 
     relationships = [
         dict(
-            rId='rId0',
-            rType='foo/hyperlink',
+            relationship_id='rId0',
+            relationship_type='foo/hyperlink',
             external=True,
             target_path='www.google.com',
         ),
@@ -119,8 +126,8 @@ class HyperlinkVanillaTestCase(_TranslationTestCase):
 class HyperlinkWithMultipleRunsTestCase(_TranslationTestCase):
     relationships = [
         dict(
-            rId='rId0',
-            rType='foo/hyperlink',
+            relationship_id='rId0',
+            relationship_type='foo/hyperlink',
             external=True,
             target_path='www.google.com',
         ),
@@ -142,8 +149,8 @@ class HyperlinkWithMultipleRunsTestCase(_TranslationTestCase):
 class HyperlinkNoTextTestCase(_TranslationTestCase):
     relationships = [
         dict(
-            rId='rId0',
-            rType='foo/hyperlink',
+            relationship_id='rId0',
+            relationship_type='foo/hyperlink',
             external=True,
             target_path='www.google.com',
         ),
@@ -178,8 +185,8 @@ class HyperlinkWithoutDefinedRelationshipDictTestCase(_TranslationTestCase):
 class HyperlinkWithBreakTestCase(_TranslationTestCase):
     relationships = [
         dict(
-            rId='rId0',
-            rType='foo/hyperlink',
+            relationship_id='rId0',
+            relationship_type='foo/hyperlink',
             external=True,
             target_path='www.google.com',
         ),
@@ -200,18 +207,18 @@ class HyperlinkWithBreakTestCase(_TranslationTestCase):
 class ImageLocal(_TranslationTestCase):
     relationships = [
         dict(
-            rId='rId0',
-            rType='foo/image',
+            relationship_id='rId0',
+            relationship_type=ImagePart.relationship_type,
             external=False,
-            target_path='word/media/image1.jpeg',
-            target=OPCPart(raw_data='content1'),
+            target_path='media/image1.jpeg',
+            data=b'content1',
         ),
         dict(
-            rId='rId1',
-            rType='foo/image',
+            relationship_id='rId1',
+            relationship_type=ImagePart.relationship_type,
             external=False,
-            target_path='word/media/image2.jpeg',
-            target=OPCPart(raw_data='content2'),
+            target_path='media/image2.jpeg',
+            data=b'content2',
         ),
     ]
 
@@ -227,29 +234,28 @@ class ImageLocal(_TranslationTestCase):
             drawing,
             pict,
         ]
-        body = ''
+        body = b''
         for el in tags:
             body += el
 
-        xml = DXB.xml(body)
-        return xml
+        return DXB.xml(body)
 
 
 class ImageTestCase(_TranslationTestCase):
     relationships = [
         dict(
-            rId='rId0',
-            rType='foo/image',
+            relationship_id='rId0',
+            relationship_type=ImagePart.relationship_type,
             external=False,
-            target_path='word/media/image1.jpeg',
-            target=OPCPart(raw_data='content1'),
+            target_path='media/image1.jpeg',
+            data=b'content1',
         ),
         dict(
-            rId='rId1',
-            rType='foo/image',
+            relationship_id='rId1',
+            relationship_type=ImagePart.relationship_type,
             external=False,
-            target_path='word/media/image2.jpeg',
-            target=OPCPart(raw_data='content2'),
+            target_path='media/image2.jpeg',
+            data=b'content2',
         ),
     ]
 
@@ -271,7 +277,7 @@ class ImageTestCase(_TranslationTestCase):
             drawing,
             pict,
         ]
-        body = ''
+        body = b''
         for el in tags:
             body += el
 
@@ -337,8 +343,8 @@ class ImageNotInRelsDictTestCase(_TranslationTestCase):
 class ImageNoSizeTestCase(_TranslationTestCase):
     relationships = [
         dict(
-            rId='rId0',
-            rType='foo/image',
+            relationship_id='rId0',
+            relationship_type=ImagePart.relationship_type,
             external=False,
             target_path=os.path.join(
                 os.path.abspath(os.path.dirname(__file__)),
@@ -346,7 +352,7 @@ class ImageNoSizeTestCase(_TranslationTestCase):
                 'fixtures',
                 'bullet_go_gray.png',
             ),
-            target=OPCPart(raw_data='content1'),
+            data='content1',
         )
     ]
     image_sizes = {
@@ -372,7 +378,7 @@ class ImageNoSizeTestCase(_TranslationTestCase):
         tags = [
             drawing,
         ]
-        body = ''
+        body = b''
         for el in tags:
             body += el
 
@@ -524,7 +530,7 @@ class TableWithListAndParagraph(_TranslationTestCase):
             ('AAA', 0, 1),
             ('BBB', 0, 1),
         ]
-        lis = ''
+        lis = b''
         for text, ilvl, numId in li_text:
             lis += DXB.li(text=text, ilvl=ilvl, numId=numId)
         els = [
@@ -532,7 +538,7 @@ class TableWithListAndParagraph(_TranslationTestCase):
             DXB.p_tag('CCC'),
             DXB.p_tag('DDD'),
         ]
-        td = ''
+        td = b''
         for el in els:
             td += el
         cell1 = DXB.table_cell(td)
@@ -565,7 +571,7 @@ class SimpleListTestCase(_TranslationTestCase):
             ('BBB', 0, 1),
             ('CCC', 0, 1),
         ]
-        lis = ''
+        lis = b''
         for text, ilvl, numId in li_text:
             lis += DXB.li(text=text, ilvl=ilvl, numId=numId)
 
@@ -591,7 +597,7 @@ class SingleListItemTestCase(_TranslationTestCase):
         li_text = [
             ('AAA', 0, 1),
         ]
-        lis = ''
+        lis = b''
         for text, ilvl, numId in li_text:
             lis += DXB.li(text=text, ilvl=ilvl, numId=numId)
 
@@ -633,7 +639,7 @@ class ListWithContinuationTestCase(_TranslationTestCase):
             table,
             DXB.li(text='HHH', ilvl=0, numId=1),
         ]
-        body = ''
+        body = b''
         for el in tags:
             body += el
 
@@ -673,7 +679,7 @@ class ListWithMultipleContinuationTestCase(_TranslationTestCase):
             table2,
             DXB.li(text='DDD', ilvl=0, numId=1),
         ]
-        body = ''
+        body = b''
         for el in tags:
             body += el
 
@@ -687,11 +693,10 @@ class MangledIlvlTestCase(_TranslationTestCase):
             <li>AAA</li>
         </ol>
         <ol list-style-type="decimal">
-            <li>BBB
-                <ol list-style-type="decimal">
-                    <li>CCC</li>
-                </ol>
-            </li>
+            <li>BBB</li>
+        </ol>
+        <ol list-style-type="decimal">
+            <li>CCC</li>
         </ol>
     '''
 
@@ -701,7 +706,7 @@ class MangledIlvlTestCase(_TranslationTestCase):
             ('BBB', 1, 1),
             ('CCC', 0, 1),
         ]
-        lis = ''
+        lis = b''
         for text, ilvl, numId in li_text:
             lis += DXB.li(text=text, ilvl=ilvl, numId=numId)
 
@@ -731,7 +736,7 @@ class SeperateListsTestCase(_TranslationTestCase):
             ('BBB', 0, 1),
             ('CCC', 0, 2),
         ]
-        lis = ''
+        lis = b''
         for text, ilvl, numId in li_text:
             lis += DXB.li(text=text, ilvl=ilvl, numId=numId)
 
@@ -744,14 +749,13 @@ class InvalidIlvlOrderTestCase(_TranslationTestCase):
         <ol list-style-type="decimal">
             <li>AAA
                 <ol list-style-type="decimal">
-                    <li>BBB
-                        <ol list-style-type="decimal">
-                            <li>CCC</li>
-                        </ol>
-                    </li>
+                    <li>BBB</li>
                 </ol>
             </li>
         </ol>
+        <ol list-style-type="decimal">
+            <li>CCC</li>
+        </ol>
     '''
 
     def get_xml(self):
@@ -760,7 +764,7 @@ class InvalidIlvlOrderTestCase(_TranslationTestCase):
             DXB.li(text='BBB', ilvl=3, numId=1),
             DXB.li(text='CCC', ilvl=2, numId=1),
         ]
-        body = ''
+        body = b''
         for el in tags:
             body += el
 
@@ -793,7 +797,14 @@ class DeeplyNestedTableTestCase(_TranslationTestCase):
             end_time = time.time()
             total_time = end_time - start_time
             # This finishes in under a second on python 2.7
-            assert total_time < 3, total_time
+            expected_time = 3
+            if sys.version_info[0] == 3:
+                expected_time = 5  # Slower on python 3
+            error_message = 'Total time: %s; Expected time: %d' % (
+                total_time,
+                expected_time,
+            )
+            assert total_time < expected_time, error_message
 
 
 class LargeCellTestCase(_TranslationTestCase):
@@ -820,7 +831,14 @@ class LargeCellTestCase(_TranslationTestCase):
             end_time = time.time()
             total_time = end_time - start_time
             # This finishes in under a second on python 2.7
-            assert total_time < 3, total_time
+            expected_time = 3
+            if sys.version_info[0] == 3:
+                expected_time = 5  # Slower on python 3
+            error_message = 'Total time: %s; Expected time: %d' % (
+                total_time,
+                expected_time,
+            )
+            assert total_time < expected_time, error_message
 
 
 class NonStandardTextTagsTestCase(_TranslationTestCase):
@@ -1002,7 +1020,7 @@ class MissingIlvl(_TranslationTestCase):
             ('BBB', None, 1),  # Because why not.
             ('CCC', 0, 1),
         ]
-        lis = ''
+        lis = b''
         for text, ilvl, numId in li_text:
             lis += DXB.li(text=text, ilvl=ilvl, numId=numId)
         body = lis
@@ -1039,13 +1057,13 @@ class SameNumIdInTable(_TranslationTestCase):
         li_text = [
             ('BBB', 0, 1),
         ]
-        lis = ''
+        lis = b''
         for text, ilvl, numId in li_text:
             lis += DXB.li(text=text, ilvl=ilvl, numId=numId)
         cell1 = DXB.table_cell(lis)
         rows = DXB.table_row([cell1])
         table = DXB.table([rows])
-        lis = ''
+        lis = b''
         lis += DXB.li(text='AAA', ilvl=0, numId=1)
         lis += table
         lis += DXB.li(text='CCC', ilvl=0, numId=1)
@@ -1064,7 +1082,7 @@ class SDTTestCase(_TranslationTestCase):
     '''
 
     def get_xml(self):
-        body = ''
+        body = b''
         body += DXB.li(text='AAA', ilvl=0, numId=0)
         body += DXB.sdt_tag(p_tag=DXB.p_tag(text='BBB'))
         body += DXB.li(text='CCC', ilvl=0, numId=0)
@@ -1115,7 +1133,7 @@ class HeadingTestCase(_TranslationTestCase):
             DXB.p_tag(text='GGG', style='style5'),
             DXB.p_tag(text='HHH', style='garbage'),
         ]
-        body = ''
+        body = b''
         for tag in p_tags:
             body += tag
 
@@ -1163,7 +1181,7 @@ class StyledBoldingTestCase(_TranslationTestCase):
                 style='style0',
             ),
         ]
-        body = ''
+        body = b''
         for tag in p_tags:
             body += tag
 
@@ -1219,7 +1237,7 @@ class RomanNumeralToHeadingTestCase(_TranslationTestCase):
             ('FFF', 1, 3),
             ('GGG', 2, 3),
         ]
-        body = ''
+        body = b''
         for text, ilvl, numId in li_text:
             body += DXB.li(text=text, ilvl=ilvl, numId=numId)
 
@@ -1273,7 +1291,7 @@ class SuperAndSubScripts(_TranslationTestCase):
                 ],
             ),
         ]
-        body = ''
+        body = b''
         for p_tag in p_tags:
             body += p_tag
 
@@ -1378,7 +1396,7 @@ class AvaliableInlineTags(_TranslationTestCase):
                 ],
             ),
         ]
-        body = ''
+        body = b''
         for p_tag in p_tags:
             body += p_tag
 
@@ -1387,7 +1405,7 @@ class AvaliableInlineTags(_TranslationTestCase):
 
 
 class UnicodeTestCase(_TranslationTestCase):
-    expected_output = u"""
+    expected_output = """
         <p>\U0010001f</p>
     """
 
@@ -1402,15 +1420,15 @@ class UnicodeTestCase(_TranslationTestCase):
             ),
         ]
 
-        body = ''
+        body = b''
         for tag in tags:
             body += tag
         xml = DXB.xml(body)
-        return xml.encode('utf-8')
+        return xml
 
 
 class NoTextInTTagTestCase(_TranslationTestCase):
-    expected_output = u"""
+    expected_output = """
     """
 
     def get_xml(self):
@@ -1424,8 +1442,43 @@ class NoTextInTTagTestCase(_TranslationTestCase):
             ),
         ]
 
-        body = ''
+        body = b''
         for tag in tags:
             body += tag
-        xml = DXB.xml(body)
-        return xml.encode('utf-8')
+        return DXB.xml(body)
+
+
+class NestedListTestCase(_TranslationTestCase):
+    expected_output = u"""
+    <ol list-style-type="decimal">
+        <li>AAA
+            <ol list-style-type="decimal">
+                <li>BBB</li>
+            </ol>
+        </li>
+        <li>CCC
+            <ol list-style-type="decimal">
+                <li>DDD
+                    <ol list-style-type="decimal">
+                        <li>EEE</li>
+                    </ol>
+                </li>
+            </ol>
+        </li>
+    </ol>
+    """
+
+    def get_xml(self):
+        li_text = [
+            ('AAA', 0, 1),
+            ('BBB', 1, 1),
+            ('CCC', 0, 1),
+            ('DDD', 1, 1),
+            ('EEE', 2, 1),
+        ]
+        lis = b''
+        for text, ilvl, numId in li_text:
+            lis += DXB.li(text=text, ilvl=ilvl, numId=numId)
+
+        xml = DXB.xml(lis)
+        return xml
