@@ -1,3 +1,9 @@
+from __future__ import (
+    absolute_import,
+    print_function,
+    unicode_literals,
+)
+
 import re
 import zipfile
 from contextlib import contextmanager
@@ -65,23 +71,36 @@ def el_iter(el):
     Go through all elements
     """
     try:
-        return el.iter()
+        for child in el.iter():
+            yield child
     except AttributeError:
-        return el.findall('.//*')
+        # iter isn't available in < python 2.7
+        for child in el.getiterator():
+            yield child
 
 
 def find_first(el, tag):
     """
     Find the first occurrence of a tag beneath the current element.
     """
-    return el.find('.//' + tag)
+    search_path = './/' + tag
+    # Due to a bug in python 2.6's ElementPath implementation, we have to
+    # strictly pass in a string
+    # https://mail.python.org/pipermail/python-bugs-list/2008-July/056209.html
+    search_path = str(search_path)
+    return el.find(search_path)
 
 
 def find_all(el, tag):
     """
     Find all occurrences of a tag
     """
-    return el.findall('.//' + tag)
+    search_path = './/' + tag
+    # Due to a bug in python 2.6's ElementPath implementation, we have to
+    # strictly pass in a string
+    # https://mail.python.org/pipermail/python-bugs-list/2008-July/056209.html
+    search_path = str(search_path)
+    return el.findall(search_path)
 
 
 def find_ancestor_with_tag(pre_processor, el, tag):
@@ -110,25 +129,13 @@ def _filter_children(element, tags):
     ]
 
 
-def remove_namespaces(document):
+def remove_namespaces(xml_bytes):
     """
-    >>> exception_raised = False
-    >>> try:
-    ...     remove_namespaces('junk')
-    ... except MalformedDocxException:
-    ...     exception_raised = True
-    >>> assert exception_raised
+    Given a stream of xml bytes, strip all namespaces from tag and attribute
+    names.
     """
-    encoding_regex = re.compile(
-        r'<\?xml.*encoding="(.+?)"',
-        re.DOTALL | re.MULTILINE,
-    )
-    encoding = 'us-ascii'
-    m = encoding_regex.match(document)
-    if m:
-        encoding = m.groups(0)[0]
     try:
-        root = cElementTree.fromstring(document)
+        root = cElementTree.fromstring(xml_bytes)
     except SyntaxError:
         raise MalformedDocxException('This document cannot be converted.')
     for child in el_iter(root):
@@ -137,7 +144,10 @@ def remove_namespaces(document):
             (k.split("}")[-1], v)
             for k, v in child.attrib.items()
         )
-    return cElementTree.tostring(root, encoding=encoding)
+    # Regardless of whatever the original encoding was
+    # (fromstring deals with it for us), always deal in terms of utf-8
+    # internally.
+    return cElementTree.tostring(root, encoding='utf-8')
 
 
 def get_list_style(numbering_root, num_id, ilvl):
@@ -175,6 +185,9 @@ class NamespacedNumId(object):
             self._num_tables,
         )
 
+    def __str__(self, *args, **kwargs):
+        return self.__unicode__(*args, **kwargs)
+
     def __repr__(self, *args, **kwargs):
         return self.__unicode__(*args, **kwargs)
 
@@ -192,6 +205,9 @@ class NamespacedNumId(object):
     def num_id(self):
         return self._num_id
 
+    def __hash__(self):
+        return id(str(self))
+
 
 class PydocxPreProcessor(MulitMemoizeMixin):
     def __init__(
@@ -217,6 +233,7 @@ class PydocxPreProcessor(MulitMemoizeMixin):
         self._set_is_in_table(root)
 
         body = find_first(root, 'body')
+        self._set_next(body)
         p_elements = [
             child for child in find_all(body, 'p')
         ]
@@ -232,7 +249,6 @@ class PydocxPreProcessor(MulitMemoizeMixin):
 
         self._set_headers(p_elements)
         self._convert_upper_roman(body)
-        self._set_next(body)
 
     def is_first_list_item(self, el):
         return self.meta_data[el].get('is_first_list_item', False)
@@ -326,6 +342,9 @@ class PydocxPreProcessor(MulitMemoizeMixin):
         # Lists are grouped by having the same `num_id` and `ilvl`. The first
         # list item is the first list item found for each `num_id` and `ilvl`
         # combination.
+        if not ilvls:
+            return
+        lowest_ilvl = min(int(i) for i in ilvls)
         for num_id in num_ids:
             for ilvl in ilvls:
                 filtered_list_elements = [
@@ -337,8 +356,24 @@ class PydocxPreProcessor(MulitMemoizeMixin):
                 ]
                 if not filtered_list_elements:
                     continue
-                first_el = filtered_list_elements[0]
-                self.meta_data[first_el]['is_first_list_item'] = True
+                # The root list needs to be handled a little differently. We
+                # only care about the first element in the root list.
+                if int(ilvl) == lowest_ilvl:
+                    filtered_list_elements = [filtered_list_elements[0]]
+
+                first_one_marked = False
+                for el in filtered_list_elements:
+                    prev_el = self.previous(el)
+                    if prev_el is None:
+                        self.meta_data[el]['is_first_list_item'] = True
+
+                    if self.ilvl(prev_el) != self.ilvl(el):
+                        self.meta_data[el]['is_first_list_item'] = True
+                    # The first list element in filtered_list_elements is
+                    # always the first list item, no matter what.
+                    if not first_one_marked:
+                        self.meta_data[el]['is_first_list_item'] = True
+                        first_one_marked = True
 
     def _set_last_list_item(self, num_ids, list_elements):
         # Find last list elements. Only mark list tags as the last list tag if
@@ -497,43 +532,15 @@ def parse_xml_from_string(xml):
 
 
 def convert_dictionary_to_style_fragment(style):
-    items = sorted(style.iteritems())
+    items = sorted(style.items())
     return ';'.join("%s:%s" % item for item in items)
 
 
 def convert_dictionary_to_html_attributes(attributes):
-    items = sorted(attributes.iteritems())
+    items = sorted(attributes.items())
     return ' '.join('%s="%s"' % item for item in items)
 
 
-def zip_path_join(*parts):
-    '''
-    Internally, ZipFile stores forward slashes as required by the zip
-    file specification. This occurs REGARDLESS of the operating
-    system, so we cannot use os.path.join.
-    See http://www.pkware.com/documents/casestudies/APPNOTE.TXT
-    Section 4.4.17
-
-    >>> zip_path_join()
-    ''
-    >>> zip_path_join('')
-    ''
-    >>> zip_path_join('', 'foo', '')
-    'foo'
-    >>> zip_path_join('foo')
-    'foo'
-    >>> zip_path_join('foo', 'bar')
-    'foo/bar'
-    >>> zip_path_join('', '', 'foo', 'bar', '',)
-    'foo/bar'
-    >>> zip_path_join(1, 2, 3)
-    '1/2/3'
-    '''
-    return '/'.join([
-        str(part) for part in parts if part
-    ])
-
-
 @contextmanager
 def ZipFile(path):  # This is not needed in python 3.2+
     try:
@@ -542,3 +549,13 @@ def ZipFile(path):  # This is not needed in python 3.2+
         raise MalformedDocxException('Passed in document is not a docx')
     yield f
     f.close()
+
+
+def xml_tag_split(tag):
+    '''
+    Given a xml node tag, return the namespace and the tag name. The namespace
+    is optional and will be None if not present.
+    '''
+    m = re.match('({([^}]+)})?(.+)', tag)
+    groups = m.groups()
+    return groups[1], groups[2]
